﻿using System;
using System.Linq;
using System.Text;
using System.Threading;
using System.Reflection;
using System.Reflection.Emit;
using System.Collections.Generic;
using System.Windows.Forms;
using System.ComponentModel;

namespace Esempio5
{
    static class Steps
    {
        #region Step1

        public static void Step1(Panel panel)
        {
            TextBox textBox = panel.CreateTextBox();

            Type t1 = typeof(System.String);
            Type t2 = Type.GetType("System.String");

            //Type t1 = typeof(System.Xml.XmlDocument);
            //Type t2 = Type.GetType("System.Xml.XmlDocument");
            //  The assembly-qualified name of the type to get.
            //  If the type is in the currently executing assembly or in mscorlib.dll, it is sufficient to supply the type name qualified by its namespace.
            //Type t1 = Type.GetType("System.Xml.XmlDocument, System.Xml, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");

            textBox.WriteLine("Type t1 = {0}", t1);
            textBox.WriteLine("Type t2 = {0}", t2);
            textBox.WriteLine("Type t1.Name = {0}", t1.Name);
            textBox.WriteLine("Type t1.FullName = {0}", t1.FullName);
            textBox.WriteLine("Type t1.BaseType = {0}", t1.BaseType);
            textBox.WriteLine("Type t1.AssemblyQualifiedName = {0}", t1.AssemblyQualifiedName);
            textBox.WriteLine("Type t1.Assembly = {0}", t1.Assembly);
            textBox.WriteLine("Type t1.Assembly.Location = {0}", t1.Assembly.Location);
            textBox.WriteLine("Type t1.Module.Name = {0}", t1.Module.Name);
            textBox.WriteLine("Type t1.Module.FullyQualifiedName = {0}", t1.Module.FullyQualifiedName);
        }

        #endregion

        #region Step2

        public static void Step2(Panel panel)
        {
            TreeView treeView = panel.CreateTreeView();

            treeView.Display(typeof(int));
            treeView.Display(typeof(string));
            treeView.Display(typeof(Type));
            treeView.Display(typeof(System.Xml.XmlDocument));
        }

        #endregion

        #region Step3

        public static void Step3(Panel panel)
        {
            TreeView treeView = panel.CreateTreeView();

            //  ATTENZIONE: copiare il file PhoenixGrid.dll nella cartella bin\debug
            Assembly assembly = Assembly.Load("PhoenixGrid");
            //Assembly assembly = Assembly.GetAssembly(typeof(Steps));
            //Assembly assembly = Assembly.GetExecutingAssembly();

            foreach (Module module in assembly.GetModules())
            {
                foreach (Type type in module.GetTypes())
                {
                    treeView.Display(type);
                }
            }

            #region Alternativa

            //foreach (Type type in assembly.GetTypes())
            //{
            //    treeView.Display(type);
            //}

            #endregion
        }

        #endregion

        #region Step3bis

        public static void Step3bis(Panel panel)
        {
            TextBox textBox = panel.CreateTextBox();

            //  Per avere l'elenco di tutti gli assembly caricati correntemente:
            foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
            {
                textBox.WriteLine(assembly);
            }
            textBox.WriteLine();

            //  Per caricare un assembly:
            //  L'argomento è il nome dell'assembly
            //    the file name of the manifest file of the assembly, minus its extension
            Assembly assembly1 = Assembly.Load("PhoenixGrid");
            //    oppure nome completo
            Assembly assembly2 = Assembly.Load("PhoenixGrid, Version=1.0.1796.17199, Culture=neutral, PublicKeyToken=null");
            //  L'argomento è il nome del file - il nome è relativo al direttorio corrente e deve essere completo di estensione
            Assembly assembly3 = Assembly.LoadFrom("PhoenixGrid.dll");
            Assembly assembly4 = Assembly.LoadFrom("../../PhoenixGrid.dll");

            textBox.WriteLine("assembly1.Location = {0}", assembly1.Location);
            textBox.WriteLine("assembly2.Location = {0}", assembly2.Location);
            textBox.WriteLine("assembly3.Location = {0}", assembly3.Location);
            textBox.WriteLine("assembly4.Location = {0}", assembly4.Location);
            textBox.WriteLine();

            //  Type è una classe astratta
            Type type = typeof(System.String);
            textBox.WriteLine("typeof(System.String).GetType().FullName = {0}", type.GetType().FullName);

            //  Anche PropertyInfo & co. sono classi astratte
            PropertyInfo propertyInfo = typeof(Steps).GetProperty("Testo");
            textBox.WriteLine("typeof(Steps).GetProperty(\"Testo\").GetType().FullName = {0}", propertyInfo.GetType().FullName);
        }

        #endregion

        #region Step4

        public static void Step4(Panel panel)
        {
            TextBox textBox = panel.CreateTextBox();

            const string stringa = "a*b*c*d";
            textBox.WriteLine(stringa.Replace('*', '$'));
            textBox.WriteLine(Testo);
            textBox.WriteLine();

            Type t1 = typeof(string);
            MethodInfo methodInfo = t1.GetMethod("Replace", new Type[] { typeof(char), typeof(char) });
            textBox.WriteLine(methodInfo);
            string result1 = (string) methodInfo.Invoke(stringa, new object[] { '*', '$' });
            textBox.WriteLine(result1);
            textBox.WriteLine();

            Type t2 = typeof(Steps);
            PropertyInfo propertyInfo = t2.GetProperty("Testo");
            textBox.WriteLine(propertyInfo);
            string result2 = (string) propertyInfo.GetValue(null, null);
            textBox.WriteLine(result2);
            textBox.WriteLine();
        }

        public static string Testo
        {
            get { return "Testo di prova esempio5"; }
        }

        #endregion

        #region Step4bis

        public static void Step4bis(Panel panel)
        {
            TextBox textBox = panel.CreateTextBox();

            List<Studente> studenti = new List<Studente>();
            studenti.Add(CreateStudente(101010, "Pallino", "Pinco", "Ingegneria Informatica", new DateTime(1977, 7, 7), 1998));
            studenti.Add(CreateStudente(10101, "Lamoto", "Tofuso", "Ingegneria Meccanica", new DateTime(1966, 6, 6), 1988));
            studenti.Add(CreateStudente(11, "Fantozzi", "Ugo", "Ingegneria Alimentare", new DateTime(1936, 12, 28), 1965));
            studenti.Add(CreateStudente(888888, "Sbagliato", "Sbaglio", "Ingegneria dell'Ingegneria", new DateTime(1922, 10, 22), 2005));
            studenti.Add(CreateStudente(12, "Fantozzi", "Mariangela", "Ingegneria Elettronica", new DateTime(1969, 2, 24), 1989));

            foreach (Studente studente in studenti)
            {
                textBox.Display(studente);
                textBox.WriteLine("--------------------");
            }
            textBox.Display("Pippo");
            textBox.WriteLine("--------------------");
        }

        private static Studente CreateStudente(int matricola, string cognome, string nome,
            string corsoDiStudio, DateTime dataDiNascita, int annoDiIscrizione)
        {
            return new Studente(matricola)
            {
                Cognome = cognome,
                Nome = nome,
                CorsoDiStudio = corsoDiStudio,
                DataDiNascita = dataDiNascita,
                AnnoDiIscrizione = annoDiIscrizione
            };
        }

        #endregion

        #region Step5

        public static void Step5(Panel panel)
        {
            TextBox textBox = panel.CreateTextBox();

            string str = new String('*', 5);
            textBox.WriteLine(str);
            textBox.WriteLine(str.Replace('*', '$'));
            textBox.WriteLine();

            Type t1 = typeof(string);
            //Type t1 = Type.GetType("System.String");
            object str1 = Activator.CreateInstance(t1, new object[] { '*', 5 });
            textBox.WriteLine(str1);
            MethodInfo methodInfo = t1.GetMethod("Replace", new Type[] { typeof(char), typeof(char) });
            textBox.WriteLine(methodInfo);
            object result1 = methodInfo.Invoke(str1, new object[] { '*', '$' });
            textBox.WriteLine(result1);
            textBox.WriteLine();
        }

        #endregion

        #region Step5bis

        public static void Step5bis(Panel panel)
        {
            OperationsView operationView = new OperationsView(OperationsDescriptorsProvider.GetOperationsDescriptors());
            operationView.Dock = DockStyle.Fill;
            panel.Controls.Clear();
            panel.Controls.Add(operationView);
        }

        #endregion

        #region Step5ter

        public static void Step5ter(Panel panel)
        {
            TextBox textBox = panel.CreateTextBox();

            Type type1 = typeof(Valore<bool>);
            textBox.WriteLine(type1.FullName);
            Valore<bool> obj1 = (Valore<bool>) Activator.CreateInstance(type1);
            textBox.WriteLine(obj1.Default);
            textBox.WriteLine();

            //Type type2 = typeof(Valore<>);
            //textBox.WriteLine(type2.FullName);
            //type2 = type2.MakeGenericType(typeof(bool));
            //textBox.WriteLine(type2.FullName);
            //Valore<bool> obj2 = (Valore<bool>) Activator.CreateInstance(type2);
            //textBox.WriteLine(obj2.Default);
            //textBox.WriteLine();
        }

        #endregion

        #region Step6a

        [Description("Visualizza gli attributi custom dell'assembly in esecuzione")]
        public static void Step6a(Panel panel)
        {
            TreeView treeView = panel.CreateTreeView();

            Type attributeType = typeof(AuthorAttribute);
            Assembly assembly = Assembly.GetExecutingAssembly();
            treeView.DisplayAttributes(String.Format("Assembly {0}", assembly.GetName().Name),
                assembly.GetCustomAttributes(true));
            treeView.DisplayAttributes(String.Format("Assembly {0}", assembly.GetName().Name),
                assembly.GetCustomAttributes(attributeType, true));
        }

        #endregion

        #region Step6b

        [Description("Visualizza gli attributi custom della classe MyClass e dei suoi metodi")]
        public static void Step6b(Panel panel)
        {
            TreeView treeView = panel.CreateTreeView();

            Type type = typeof(MyClass);
            Type attributeType = typeof(AuthorAttribute);
            treeView.DisplayAttributes(String.Format("Class {0}", type.Name),
                type.GetCustomAttributes(attributeType, true));
            //  Attributi del metodo Fun1
            MethodInfo methodInfo = type.GetMethod("Fun1", Type.EmptyTypes);
            treeView.DisplayAttributes(String.Format("Method {0}.{1}", type.Name, methodInfo.Name),
                methodInfo.GetCustomAttributes(attributeType, true));
            //  Attributi del metodo Fun2
            methodInfo = type.GetMethod("Fun2", Type.EmptyTypes);
            treeView.DisplayAttributes(String.Format("Method {0}.{1}", type.Name, methodInfo.Name),
                methodInfo.GetCustomAttributes(attributeType, true));
        }

        #endregion

        #region Step6c

        [Description("Visualizza gli attributi custom della classe TextBox e della sua proprietà CharacterCasing")]
        public static void Step6c(Panel panel)
        {
            TreeView treeView = panel.CreateTreeView();

            Type type = typeof(TextBox);
            treeView.DisplayAttributes(String.Format("Class {0}", type.Name),
                type.GetCustomAttributes(true));
            //  Attributi della proprietà CharacterCasing di TextBox
            PropertyInfo propertyInfo = type.GetProperty("CharacterCasing");
            treeView.DisplayAttributes(String.Format("Property {0}.{1}", type.Name, propertyInfo.Name),
                propertyInfo.GetCustomAttributes(true));
        }

        #endregion

        #region Step7

        public static void Step7(Panel panel)
        {
            TreeView treeView = panel.CreateTreeView();

            Type myDynamicType = CreateDynamicType();
            treeView.Display(myDynamicType);
            object o = Activator.CreateInstance(myDynamicType);
            MethodInfo myMethod = myDynamicType.GetMethod("MyMethod");
            myMethod.Invoke(o, null);
        }

        private static Type CreateDynamicType()
        {
            AssemblyName myAssemblyName = new AssemblyName();
            myAssemblyName.Name = "MyAssembly";
            AssemblyBuilder myAssemblyBuilder =
                Thread.GetDomain().DefineDynamicAssembly(myAssemblyName, AssemblyBuilderAccess.RunAndSave);
            ModuleBuilder myModuleBuilder =
                myAssemblyBuilder.DefineDynamicModule("MyModule", "MyAssembly.dll");
            TypeBuilder myTypeBuilder =
                myModuleBuilder.DefineType("Esempio5.MyType", TypeAttributes.Public);
            MethodBuilder myMethodBuilder =
                // DefineMethod(string name, MethodAttributes attributes, Type returnType, Type[] parameterTypes
                myTypeBuilder.DefineMethod("MyMethod", MethodAttributes.Public, null, null);
            ILGenerator myILGenerator = myMethodBuilder.GetILGenerator();
            myILGenerator.EmitWriteLine("Hello World!");
            myILGenerator.Emit(OpCodes.Ret);
            Type myType = myTypeBuilder.CreateType();
            myAssemblyBuilder.Save("MyAssembly.dll");
            return myType;
        }

        #endregion
    }

    public class Valore<T>
    {
        public T Default
        {
            get { return default(T); }
        }
    }
}
