﻿using System;
using System.Linq;
using System.Text;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Reflection;

namespace Esempio5
{
    public static class TreeViewExtensions
    {
        public static void Display(this TreeView treeView, Type type)
        {
            TreeNode typeTreeNode = new TreeNode(type.FullName);
            treeView.Nodes.Add(typeTreeNode);

            typeTreeNode.Display("Fields", type.GetFields());
            typeTreeNode.Display("Events", type.GetEvents());
            typeTreeNode.Display("Constructors", type.GetConstructors());
            typeTreeNode.Display("Properties", type.GetProperties());
            typeTreeNode.Display("Methods", type.GetMethods());
            typeTreeNode.Display("Nested types", type.GetNestedTypes());
        }

        private static void Display(this TreeNode parentTreeNode, string name, MemberInfo[] members)
        {
            TreeNode treeNode = new TreeNode(name);
            parentTreeNode.Nodes.Add(treeNode);
            foreach (MemberInfo memberInfo in members)
                treeNode.Nodes.Add(memberInfo.ToString());
        }

        public static void DisplayAttributes(this TreeView treeView, string title, object[] attributes)
        {
            if (attributes.Length == 0)
                return;
            TreeNode firstTreeNode = new TreeNode(title);
            treeView.Nodes.Add(firstTreeNode);

            foreach (Attribute attribute in attributes)
            {
                Type attributeType = attribute.GetType();
                TreeNode attributeTreeNode = new TreeNode(attributeType.Name);
                firstTreeNode.Nodes.Add(attributeTreeNode);
                foreach (PropertyInfo propertyInfo in attributeType.GetProperties())
                {
                    if (propertyInfo.Name == "TypeId")
                        continue;
                    object value = propertyInfo.GetValue(attribute, null);
                    attributeTreeNode.Nodes.Add(String.Format("{0}=\"{1}\" ", propertyInfo.Name, value));
                }
            }
        }
    }
}
